/*****************************************************************************
*
*	File		ReadWriteSocket.java
*	Author		Vijay Sekhri
*	Date		Tue Apr  8 11:49:15 CDT 2003
*	Organization	Fermi National Accelerator Laboratory
*	Description	Methods to read and write from the socket stream
*	Email		sekhri@fnal.gov
*
*
******************************************************************************/
package gov.fnal.vox.saz.bl.util;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.InputStream;
import java.io.PrintStream;
import java.io.BufferedReader;
import java.io.OutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.net.Socket;
import java.util.Date;

import org.apache.log4j.Logger;

import org.globus.gsi.gssapi.SSLUtil;

public class ReadWriteSocket{
	private BufferedReader reader=null;
	private PrintStream writer=null;
	private InputStream in=null;
	private OutputStream out=null;
	private DataInputStream din=null;
	private DataOutputStream dout=null;
	private Socket clientSocket=null;
	private int MAXLENGTH=16000;
	private static final transient Logger logger=Logger.getLogger(ReadWriteSocket.class.getName());	

	/**
	* An object for reading and writing to the socket.
	* @param clientSocket The Socket on which read and write operation can be performed.
	*/
	public ReadWriteSocket(Socket clientSocket) throws Exception{
		this.clientSocket=clientSocket;
		try{
			logger.debug("Before getting OutputStream");
                        out=clientSocket.getOutputStream();
			logger.debug("Before getting InputStream");
			in=clientSocket.getInputStream();
			reader =new BufferedReader(new InputStreamReader(in));
			logger.debug("After getting buffered reader");
			writer=new PrintStream(out);
			logger.debug("After getting writer");
			din=new DataInputStream(in);
			logger.debug("After getting DataInputStream");
			dout=new DataOutputStream(out);
			logger.debug("After getting DataOutputStream");
			if((in == null)||(out ==null)||(reader ==null)||(writer==null)||(din==null)||(dout ==null)){
				throw new Exception("Exception: Invalid client");
			}
		}catch(IOException e){
			logger.error(e.getMessage());
		}
	}


	/**
	* A method for reading a line from the socket.
	* @return A String read from the socket input stream.
	*/
	public String readLine() throws Exception{
		String message="";
		message=reader.readLine();
		logger.debug("Received ---> "+message);
		return(message);
	}

	/**
	* A method for writing a line on the socket.
	* @param message A String to be written on to the socket output stream.
	*/
	public void writeLine(String message) throws Exception{
		writer.println(message);
		logger.debug("Sent ---> "+message);
	}

	public void writeToken(byte[] outToken) throws Exception{
		out.write(outToken);
		out.flush();
	}

	public void sendMessage(String message) throws Exception{
		byte outToken[]=message.getBytes();
		out.write(outToken);
		out.flush();
	}

	public String recvMessage() throws Exception{
		String line="";
		char buf[]=new char[MAXLENGTH];
		int len=reader.read(buf,0,MAXLENGTH-1);
		char toReturn[]=new char[len];
		for(int i=0;i<len;i++){
			toReturn[i]=buf[i];
		}
		return(String.valueOf(toReturn));
	}

	public byte[] readToken() throws Exception{
		return(SSLUtil.readSslMessage(in));
	}

	public byte[] readKerberosToken() throws Exception{
		byte[] token=new byte[din.readInt()];
		din.readFully(token);
		return(token);
	}
	
	public void writeKerberosToken(byte[] token) throws Exception{
		dout.writeInt(token.length);
		dout.write(token);
		dout.flush();
	}

	/**
	* A method for closing both the input and output stream of the socket.
	*/
	public void closeAll(){
		try{
			if(reader != null) reader.close();
			if(writer != null) writer.close();
			if(din != null) din.close();
			if(dout != null) dout.close();

		}catch(IOException e){
			logger.error("Exception: "+e.getMessage());
		}

	}
}
