/*****************************************************************************
*
*	File		SAZUtil.java
*	Author		Vijay Sekhri, Neha Sharma
*	Date		Tue Nov 16 16:37:18 CST 2004
*	Organization	Fermi National Accelerator Laboratory
*	Description	All the neccessary functions used everywhere
*	Email		sekhri@fnal.gov
*
*
******************************************************************************/
package gov.fnal.vox.saz.bl.util;

import java.util.StringTokenizer;
import java.util.regex.Pattern;
import java.util.regex.Matcher;
import gov.fnal.vox.saz.bl.util.SAZConfiguration;

public class SAZUtil {
	public SAZUtil(){
	}
	/**
	* To remove all , from DN and replace them with /
	*/
	public String changeDN(String dn){
		StringTokenizer tokenizer=new StringTokenizer(dn,",");
		String temp="";
		if(tokenizer.countTokens()>1){
			temp="/"+tokenizer.nextToken().trim();
			while(tokenizer.hasMoreTokens()){
				temp=temp+"/"+tokenizer.nextToken().trim();
			}
			return(removeProxyFromDN(temp));
		}else{
			return(removeProxyFromDN(dn));
		}
	}
	public String removeProxyFromDN(String dn){
		int proxyIndex=dn.indexOf("/CN=proxy");
		if(proxyIndex==-1){		
			return(dn);
		}
		return(dn.substring(0,proxyIndex));
	}
	/* Neha  - 03/12 Function to strip CN=random-number field present in the DN 
		 - 09/24 Support DN of form "/DC=ch/DC=cern/OU=Organic Units/OU=Users/CN=giunta/CN=531496/CN=Marina Giunta"
		   where CN=some-random-number is followed by CN=something-that-makes-sense. This was previously being stripped 
	*/
	public String stripRandomNumberInDN(String dn){
		int lastCNIndex=0;
                while(dn.lastIndexOf("/CN=") != -1){
                        lastCNIndex=dn.lastIndexOf("/CN=");
                        if(Character.isDigit(dn.charAt(lastCNIndex+4))){
                                dn=dn.substring(0,lastCNIndex);
                        }else{
                                break;
                        }
                }
		return dn;
	}
	/* Neha - 04/21/2011 Function to validate the User and CA DN
	*/	
	public void validateDN(String dn) throws Exception{
		Pattern dnPat;
		Matcher dnMat;
		boolean isValid;
		if((dn == null) || (dn.equals("NULL"))){
			throw new Exception("SAZUtil: EXCEPTION: DN cannot be null/NULL");
		}else{
			//dnPat=Pattern.compile("/(\\w+/?\\.?-?=?@?\\(?\\)?_?:?'?)+");
			dnPat=Pattern.compile("/(\\w?/?\\.?-?=?@?\\(?\\)?_?:?'?\\s?)+");
			dnMat=dnPat.matcher(dn);
			isValid=dnMat.matches();
			if(!isValid){
				throw new Exception("SAZUtil: EXCEPTION: DN is invalid");
			}
		}
	}
	/* Neha - 04/21/2011 Function to validate the VO Name
        */
        public void validateVO(String vo) throws Exception{
		Pattern dnPat;
                Matcher dnMat;
                boolean isValid;
		//If VO is null, we set it to NULL
                if((vo != null) && (!vo.equals("/(\\w+/?\\.?-?=?@?\\(?\\)?_?:?'?)+"))){
			dnPat=Pattern.compile("a*b");
                        dnMat=dnPat.matcher(vo);
                        isValid=dnMat.matches();
                        if(!isValid){
                                throw new Exception("SAZUtil: EXCEPTION: VO is invalid");
                        }
                }
        }
	/* Neha - 04/21/2011 Function to validate the Role 
        */
        public void validateRole(String role) throws Exception{
		Pattern dnPat;
                Matcher dnMat;
                boolean isValid;
		//If Role is null, we set it to NULL
		if((role != null) && (!role.equals("NULL"))){
        		dnPat=Pattern.compile("/(\\w+/?\\.?-?=?@?\\(?\\)?_?:?'?)+");
                        dnMat=dnPat.matcher(role);
                        isValid=dnMat.matches();
                        if(!isValid){
                                throw new Exception("SAZUtil: EXCEPTION: Role is invalid");
                        }
		}
	}
	public boolean isNull(String toCheck){
		if(toCheck!=null){
			if(toCheck.length()==0){
				return(true);
			}else{
				return(false);
			}
		}else{
			return(true);
		}
	}
}
