/******************************************************************************
*       Author          Vijay Sekhri, Valery Sergeev, Neha Sharma
*       Organization    Fermi National Accelerator Laboratory
*       Description     This class performs the main functionality of SAZ server
*			for each incoming client request
*       Email           neha@fnal.gov
*
******************************************************************************/
package gov.fnal.vox.saz.service;

import java.io.File;
import java.util.Date;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;
import java.util.ArrayList;
import java.util.UUID;
import java.lang.String;
import java.lang.StringBuilder;
import java.net.Socket;
import java.net.InetAddress;
import java.security.cert.X509Certificate;

import org.apache.log4j.Logger;
import org.glite.voms.VOMSAttribute;
import org.glite.voms.VOMSValidator;
import org.globus.gsi.gssapi.GSSConstants;
import org.gridforum.jgss.ExtendedGSSContext;
//import org.bouncycastle.jcajce.provider.asymmetric.x509.CertificateFactory;
import org.bouncycastle.jce.provider.JDKX509CertificateFactory;

import gov.fnal.vox.saz.entity.User;
import gov.fnal.vox.saz.entity.CA;
import gov.fnal.vox.saz.entity.VO;
import gov.fnal.vox.saz.entity.Role;
import gov.fnal.vox.saz.bl.HibernateUtil;
import gov.fnal.vox.saz.bl.SAZAPIImpl;
import gov.fnal.vox.saz.bl.util.SAZUtil;
import gov.fnal.vox.saz.bl.util.ANAM;
import gov.fnal.vox.saz.bl.util.CertInputStream;
import gov.fnal.vox.saz.bl.util.ReadWriteSocket;
import gov.fnal.vox.saz.bl.util.SAZConfiguration;

public class SAZClientThread implements Runnable{	
	private long proxyLifetime;
	private String userDn;
	private String issuerDn;
	private String vo="NULL";
	private String role="NULL";
	private String userEnabled;
	private String userTrusted;
	private String caEnabled;
	private String caTrusted;
	private String voEnabled;
	private String voTrusted;
	private String roleEnabled;
	private String roleTrusted;
	private String sazDecision="N";
	private boolean voProxy;
	private Date notBefore;
	private Date notAfter;
	private Socket clientSocket;
	private SAZUtil utilObject;
	private SAZConfiguration confObject;
	private SAZAPIImpl sazapiObject;
	private List<X509Certificate> userCerts;
	private ExtendedGSSContext context;
	private ReadWriteSocket rwSocket;
	private static final transient Logger logger=Logger.getLogger(SAZClientThread.class.getName());
		
		
	public SAZClientThread(Socket clientSocket){
		this.confObject=SAZConfiguration.getInstance();
		this.clientSocket=clientSocket;
		utilObject=new SAZUtil();
		sazapiObject=new SAZAPIImpl();
	}
	public void run(){
		int numUserCerts=0;
                int iteration=0;
                int datacount=0;
		int firstIndex=-1;
		boolean connection=false;
		StringBuilder message=new StringBuilder();
                String messageStr="";
		String temp="";
		String[] usercerts;
		ANAM anamObject=new ANAM();
		CertInputStream certInputStream;
		X509Certificate tempCert;
		try{
			JDKX509CertificateFactory cf=new JDKX509CertificateFactory();
			//CertificateFactory cf=new CertificateFactory();
			context=anamObject.getGsiServerContext(confObject.getDefaultSAZServerCert(),confObject.getDefaultSAZServerKey());
			clientSocket = anamObject.getGsiServerSocket(clientSocket,context);
			InetAddress remoteHost=clientSocket.getInetAddress();
			logger.info("Received authorization request from "+remoteHost.getHostAddress()+" : "+remoteHost.getHostName());
			Thread thread=Thread.currentThread();
			int Idbegin=thread.toString().indexOf("-");
			int Idend=thread.toString().indexOf(",");
			String threadId=thread.toString().substring(Idbegin+1, Idend);
			rwSocket=new ReadWriteSocket(clientSocket);
			connection=true;
			temp=rwSocket.recvMessage();
			message.append(temp);
			//message contains the user certificate chain
			while((!temp.equals("done"))&&(datacount<8)){
				logger.debug("datacount: "+datacount);
				rwSocket.sendMessage("more");
				logger.debug("send me more data");
				temp=rwSocket.recvMessage();	
				logger.debug("received data");
				if(!temp.equals("done")){
					message=message.append(temp);
					datacount++;
				}
			}
			if(datacount==8){
				cleanup();
			}
			messageStr=message.toString();
			//logger.debug("message: "+message);
			//split the user certificate chain in to individual certs
			//for each cert, get the subject and issuer dn
			usercerts=messageStr.split("-----END CERTIFICATE-----");
			//when request comes in through command line sazclient with 
			//a proxy without any extended credentials, then number of 
			//user certificates is 1.
			// -1 because the last element is an empty cert, so no need
			//to extract anything from that location
			numUserCerts=usercerts.length;
			if(numUserCerts>1){	
				numUserCerts--;
			}
			userCerts=new ArrayList<X509Certificate>();
			logger.debug("SAZClientThread: Number of certificates in user certificate chain: "+numUserCerts);
			for(int s=0;s<numUserCerts;s++){
				usercerts[s]=usercerts[s].concat("-----END CERTIFICATE-----");
				logger.debug("SAZClientThread: usercerts["+s+"]: "+usercerts[s]);	
				certInputStream=new CertInputStream(usercerts[s]);
				tempCert=(X509Certificate)cf.engineGenerateCertificate(certInputStream);
				//userCerts[s]=tempCert;
				userCerts.add(s,tempCert);
			}
			//get extended attributes if they are present
			List listOfAttributes=getVOMSAttributeList(userCerts);
			if(!listOfAttributes.isEmpty()){
				getVOAndRole(listOfAttributes);
			}else{
				//if you reach here, either its a proxy without any extended
				//credentials or its special case in which extended attributes
				//need to be extracted from the proxy file manually
				logger.debug("SAZClientThread: Special Case: Need to get VO and Role from Proxy File");
				getVOAndRoleFromFile(threadId,messageStr);
			}
			//set the value of user and issuer dn
                        setUserAndIssuerDN();
			//userDn=userDn.replaceAll("\'","\\\\\'");	
			//prepending a / in front of Role in case it is not NULL
			if(!role.equals("NULL")){
				role="/"+role;	
			}
			//logger.info("User "+userDn);
			//logger.info("CA "+issuerDn);
			//logger.info("VO "+vo);
			//logger.info("Role "+role);
			logger.debug("SAZClientThread: Going to check if user is authorized or not...");
			//extract the user/ca/vo/role information from database or add if not
			//already present
			sazDecision = sazapiObject.isEnabled(userDn,issuerDn,vo,role);
			//send response back to user
			//If banning based on user proxy lifetime is enabled, and user proxy lifetime does exceed
                        //maximum configured, set SAZ decision to "N"
			 if(sazapiObject.getIsUserProxyLifetimeBanEnabled().equals("Y")){
                                //logger.info("Incoming Proxy Lifetime " +proxyLifetime+" seconds");
                                if(sazapiObject.compareProxyLifetime(proxyLifetime)){
                                        logger.info("Incoming Proxy Lifetime exceeds the maximum allowed by SAZ Administrator");
                                        sazDecision = "N";
                                }
                        }
			logger.info("Sending to client "+sazDecision);
			sendResponseToClient(userDn,rwSocket);
		}catch(Exception e){   
			if(!connection){
				logger.error("SAZClientThread: EXCEPTION: Could not establish connection with client!");
			}
			logger.error(e.getMessage());
		}finally{
			cleanup();
		}
	}
	//function to cleanup and dispose of all the used resources
	private void cleanup(){
		try{
			logger.info("SAZClientThread: Closing all connections... ");
			logger.debug("SAZClientThread: Closing all connections...");
			if(context != null) context.dispose();
			if(rwSocket != null) rwSocket.closeAll();
			if(clientSocket != null) clientSocket.close();
		}catch(Exception e1){
			logger.error(e1.getMessage());
			try{
				if(rwSocket != null) rwSocket.closeAll();
				if(clientSocket != null) clientSocket.close();	
			}catch(Exception e2){
				logger.error(e2.getMessage());
				try{
					if(clientSocket != null) clientSocket.close();
				}catch(Exception e3){
					logger.error(e3.getMessage());
				}
			}
		}	
	}
	//function to extract vo and role informaion
	private void getVOAndRole(List vomsAttributeList) throws Exception{
		try{
			VOMSAttribute voAttr=(VOMSAttribute) vomsAttributeList.get(0);
			vo=voAttr.getVO();
			String fqan=voAttr.getFullyQualifiedAttributes().toString();
			int fqanLength=fqan.length();
			String fqanList=null;
			//Neha - 10/24/08 Added below condition to handle case when there 
			//are no FQANs i.e. it looks like [] and hence has length of 2
			//In this case, role i.e. FQAN is set to <vo-name>/NULL
 			if(fqanLength >= 3){	
				fqanList=fqan.substring(2,fqanLength-1);
				logger.debug("fqanList: "+fqanList);
				String[] fqanSplit=fqanList.split(", ");
				role=fqanSplit[0];
 			}else{
				role=vo+"/NULL";
			}
			voProxy=true;
		}catch(Exception e ){
                       logger.error("SAZClientThread: EXCEPTION: Could not parse the extended voms attributes!");
                       throw new Exception("SAZClientThread: EXCEPTION: Could not parse the extended voms attributes!" + e.getMessage());
                }
	}
	//function to extract vo and role informaion in case of the special proxy 
	private void getVOAndRoleFromFile(String threadId,String messageStr) throws Exception{
		UUID fileid = UUID.randomUUID();
		StringBuilder filename = new StringBuilder("/tmp/proxy.");
		filename.append(fileid.toString());
		logger.debug("SAZClientThread: Proxy File: "+filename.toString());

		File proxyInfo=new File(filename.toString());
		String s=null;
		//Neha - 11/14/08 Changed cmd as with VDT-1.10.1, the voms-proxy-info returns error code of 1 instead of 0
		//String[] cmd={"/bin/sh", "-c","source /usr/local/vdt/setup.sh;"+"voms-proxy-info -all -file "+proxyInfo};
		StringBuilder command=new StringBuilder("voms-proxy-info -all -file ");
		command.append(proxyInfo);
		command.append(" | grep '.'");
		//String[] cmd={"/bin/sh", "-c","voms-proxy-info -all -file "+proxyInfo+" | grep '.'"};
		String[] cmd={"/bin/sh", "-c",command.toString()};
		try{
			logger.debug("SAZClientThread: Going to create proxy file ...");
			proxyInfo.createNewFile();
			OutputStreamWriter writer=new OutputStreamWriter(new FileOutputStream(proxyInfo));
			writer.write(messageStr,0,messageStr.length());
			writer.close();
			Process p=Runtime.getRuntime().exec(cmd,null);
			int ii=p.waitFor();	
			if(ii==0){
				BufferedReader stdInput=new BufferedReader(new InputStreamReader(p.getInputStream()));
				while((s=stdInput.readLine()) != null){
					if(s.startsWith("V")){
						String[] v=s.split(": ");
						vo=v[1];
						voProxy=true;
					}
					if(s.startsWith("attribute")){
						String r=s.split(": /")[1];
						role=r;
						voProxy=true;
					}
				}
			}else{
				BufferedReader stdErr=new BufferedReader(new InputStreamReader(p.getErrorStream()));
				while((s=stdErr.readLine()) != null){
					logger.error("SAZClientThread: EXCEPTION: Could not read the proxy file!");
				}
			}
			if((proxyInfo != null)&&(proxyInfo.exists())){
				proxyInfo.delete();
				logger.debug("Deleted the proxy file");
			}
		}catch(Exception e){
			logger.error("SAZClientThread: EXCEPTION: Could not execute \"voms-proxy-info -all\" command!");
			throw new Exception("SAZClientThread: EXCEPTION: Could not execute \"voms-proxy-info -all\" command!" + e.getMessage());
		}
	}
	private void sendResponseToClient(String dn,ReadWriteSocket rwSocket) throws Exception{
		String response="DN="+dn+"\nENABLE="+sazDecision;
		if(sazDecision.equals("Y")){
			sendMessage("yes",response,rwSocket);
		}else{
			sendMessage("no",response,rwSocket);
 		}
	}
	private void sendMessage(String status,String response,ReadWriteSocket rwSocket) throws Exception{
		rwSocket.sendMessage(status);
		String temp=rwSocket.recvMessage();
		if(temp.equals("more")){
			rwSocket.sendMessage(response);
		}
	}
	private synchronized List getVOMSAttributeList(List<X509Certificate> certs){
		List attributeList=null;	
		VOMSValidator validator=new VOMSValidator(certs.toArray(new X509Certificate[0])).parse();
                attributeList=validator.getVOMSAttributes();
		return attributeList;
	}
	//Modified to ban based on user certificate serial number if user has chosen so
	private void setUserAndIssuerDN(){
		X509Certificate[] usercertArray=userCerts.toArray(new X509Certificate[0]);	
		for(int p=0;p<userCerts.size();p++){	
			userDn=utilObject.stripRandomNumberInDN(utilObject.changeDN(usercertArray[p].getSubjectDN().getName()));
			issuerDn=utilObject.stripRandomNumberInDN(utilObject.changeDN(usercertArray[p].getIssuerDN().getName()));
			logger.debug("User DN: "+p+" :"+userDn);
			logger.debug("Issuer DN: "+p+" :"+issuerDn);
			//Instead get the dates from most recent element in certificate chain
			if(p == 0){
				notAfter=usercertArray[p].getNotAfter();
				notBefore=usercertArray[p].getNotBefore();      
				proxyLifetime=(notAfter.getTime() - notBefore.getTime())/1000;
				logger.info("NotBefore: "+notBefore+" - NotAfter: "+notAfter+" - Incoming Proxy Lifetime: " +proxyLifetime+" seconds");
			}
			if(!userDn.equals(issuerDn)){
				if(confObject.getDefaultSAZBanUserCertSerialNum().equals("Y")){
					userDn=userDn.concat("/SERIAL=");
					userDn=userDn.concat(usercertArray[p].getSerialNumber().toString());
				}
				break;
			}
		}
	}
}
