/*****************************************************************************
*
*       File            SAZServer.java
*       Author          Vijay Sekhri
*       Organization    Fermi National Accelerator Laboratory
*       Description     Main SAZ Server class 
*       Email           sekhri@fnal.gov
*
*
******************************************************************************/
package gov.fnal.vox.saz.service;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Map;
import java.util.Date;
import java.util.WeakHashMap;
import java.util.Collections;
import java.io.PrintStream;
import java.util.concurrent.Executors;
import java.util.concurrent.ExecutorService;
import org.apache.log4j.Logger;
import gov.fnal.vox.saz.bl.util.CRLRefresher;
import gov.fnal.vox.saz.bl.util.SAZConfiguration;

public class SAZServer implements Runnable{
	private ServerSocket server;
	private ExecutorService pool;
	private Socket clientSocket;
	private static final transient Logger logger=Logger.getLogger(SAZServer.class.getName());
	private final SAZConfiguration confObject=SAZConfiguration.getInstance();
	/**
	* An instance of the server which is started and forks a thread on each client connection.
	*/
	public void run(){
		try{
			confObject.loadTrustedCerts();
                        logger.debug("Done loading the Trusted Certificates/CRLs for the first time");
                        server=new ServerSocket(confObject.getDefaultSAZServerPort());
                        pool=Executors.newFixedThreadPool(confObject.getDefaultSAZServerPoolSize());
                        //thread to refresh the CRLs at configured time interval
                        logger.debug("Going to fork CRL Refresher thread");
                        new CRLRefresher();
                        logger.debug("CRL Refresher thread should now be up");
			while(true){
				try{
					clientSocket=server.accept();
          				pool.execute(new SAZClientThread(clientSocket));
				}catch(Exception e){
					logger.error(e.getMessage());
				}
        		}               
      		}catch(IOException ex){
			logger.error(ex.getMessage());	
		}catch(Exception e){
			logger.error(e.getMessage());
		}catch(Throwable t){
			logger.error(t.getMessage());
		}finally{
			try{
				pool.shutdown();
				server.close();
			}catch(IOException e){
				logger.error(e.getMessage());
			}
		}
	}
}


